@@ -1,12 +1,19 @@
 """Set of constructs for making No SQL DB Tables
 """
 from enum import Enum
-from typing import Any, FrozenSet, List
+from typing import Any, FrozenSet, List, Dict
 
-from core.constructs.resource import Resource, ResourceModel, update_hash, ResourceOutputs, PermissionsAvailableMixin
-from core.constructs.cloud_output import  Cloud_Output_Str, OutputType
+from core.constructs.resource import (
+    Resource,
+    TaggableResourceModel,
+    TaggableMixin,
+    update_hash,
+    ResourceOutputs,
+    PermissionsAvailableMixin,
+)
+from core.constructs.cloud_output import Cloud_Output_Str, OutputType
 from core.constructs.types import cdev_str_model
-from core.constructs.models import ImmutableModel
+from core.constructs.models import ImmutableModel, frozendict
 
 from core.default.resources.simple.events import Event, event_model
 from core.default.resources.simple.iam import Permission
@@ -19,7 +26,7 @@ RUUID = "cdev::simple::table"
 #################
 ##### Events
 #################
-#TODO: Add more documentation on the json schema of the events
+# TODO: Add more documentation on the json schema of the events
 class stream_type(str, Enum):
     """Type of streams for a table."""
 
@@ -41,45 +48,45 @@ class stream_event_model(event_model):
         view_type (stream_type): Type of eventthe stream produces
         batch_size (int): Size of event batches
     """
-    table_name: cdev_str_model    
+
+    table_name: cdev_str_model
     view_type: stream_type
     batch_size: int
-    batch_failure: bool 
+    batch_failure: bool
 
 
 class StreamEvent(Event):
     """Construct for representing the Stream of a `Table`"""
 
-    def __init__(self, table_name: str, view_type: stream_type, batch_size: int, batch_failure: bool= True) -> None:
+    def __init__(
+        self,
+        table_name: str,
+        view_type: stream_type,
+        batch_size: int,
+        batch_failure: bool = True,
+    ) -> None:
         """
         Args:
             table_name (str): Cdev name of the API this route is apart of
             view_type (stream_type): Type of eventthe stream produces
             batch_size (int): Size of event batches
         """
-        
+
         self.cdev_table_name = table_name
-        
-        self.table_name= Cloud_Output_Str(
-                name=table_name, 
-                ruuid=RUUID, 
-                key='table_name',
-                type=OutputType.RESOURCE 
-            )
-            
-        self.view_type=view_type
-        self.batch_size=batch_size
+
+        self.table_name = Cloud_Output_Str(
+            name=table_name, ruuid=RUUID, key="table_name", type=OutputType.RESOURCE
+        )
+
+        self.view_type = view_type
+        self.batch_size = batch_size
         self.batch_failure = batch_failure
 
-
     def hash(self) -> str:
-        return hasher.hash_list([
-            self.cdev_table_name,
-            self.view_type,
-            self.batch_size,
-            self.batch_failure
-        ])
-        
+        return hasher.hash_list(
+            [self.cdev_table_name, self.view_type, self.batch_size, self.batch_failure]
+        )
+
     def render(self) -> stream_event_model:
         return stream_event_model(
             originating_resource_name=self.cdev_table_name,
@@ -88,7 +95,7 @@ class StreamEvent(Event):
             batch_size=self.batch_size,
             view_type=self.view_type.value,
             table_name=self.table_name.render(),
-            batch_failure=self.batch_failure
+            batch_failure=self.batch_failure,
         )
 
 
@@ -108,7 +115,9 @@ class TablePermissions:
                 "dynamodb:Query",
                 "dynamodb:ConditionCheckItem",
             ],
-            cloud_id=Cloud_Output_Str(resource_name, RUUID, 'cloud_id', OutputType.RESOURCE),
+            cloud_id=Cloud_Output_Str(
+                resource_name, RUUID, "cloud_id", OutputType.RESOURCE
+            ),
             effect="Allow",
         )
         """Permissions to read data from the Table"""
@@ -127,7 +136,9 @@ class TablePermissions:
                 "dynamodb:UpdateItem",
                 "dynamodb:DescribeLimits",
             ],
-            cloud_id=Cloud_Output_Str(resource_name, RUUID, 'cloud_id', OutputType.RESOURCE),
+            cloud_id=Cloud_Output_Str(
+                resource_name, RUUID, "cloud_id", OutputType.RESOURCE
+            ),
             effect="Allow",
         )
         """Permissions to write data to the Table"""
@@ -146,7 +157,9 @@ class TablePermissions:
                 "dynamodb:Scan",
                 "dynamodb:UpdateItem",
             ],
-            cloud_id=Cloud_Output_Str(resource_name, RUUID, 'cloud_id', OutputType.RESOURCE),
+            cloud_id=Cloud_Output_Str(
+                resource_name, RUUID, "cloud_id", OutputType.RESOURCE
+            ),
             effect="Allow",
         )
         """Permissions to read and write data to and from the Table"""
@@ -159,16 +172,20 @@ class TablePermissions:
                 "dynamodb:ListShards",
                 "dynamodb:ListStreams",
             ],
-            cloud_id=Cloud_Output_Str(resource_name, RUUID, 'cloud_id', OutputType.RESOURCE).join(["", "/stream/*"]),
+            cloud_id=Cloud_Output_Str(
+                resource_name, RUUID, "cloud_id", OutputType.RESOURCE
+            ).join(["", "/stream/*"]),
             effect="Allow",
         )
         """Permissions to receive and process events from the Table Stream"""
 
+
 ##############
 ##### Output
 ##############
 class TableOutput(ResourceOutputs):
     """Container object for the returned values from the cloud after a `Table` has been deployed."""
+
     def __init__(self, name: str) -> None:
         super().__init__(name, RUUID)
 
@@ -176,10 +193,7 @@ class TableOutput(ResourceOutputs):
     def table_name(self) -> Cloud_Output_Str:
         """The name of the generated table"""
         return Cloud_Output_Str(
-            name=self._name,
-            ruuid=RUUID,
-            key='table_name',
-            type=self.OUTPUT_TYPE
+            name=self._name, ruuid=RUUID, key="table_name", type=self.OUTPUT_TYPE
         )
 
     @table_name.setter
@@ -208,7 +222,7 @@ class attribute_type(str, Enum):
 
 class key_type(str, Enum):
     """Type of key for a defined key on a table.
-    
+
     These value will be used by the primary key to determine how the data is stored in the table.
     visit https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/HowItWorks.CoreComponents.html#HowItWorks.CoreComponents.PrimaryKey for more details.
     """
@@ -226,16 +240,17 @@ class attribute_definition_model(ImmutableModel):
         attribute_name (str)
         attribute_type (`attribute_type`)
     """
-    attribute_name: str 
+
+    attribute_name: str
     attribute_type: attribute_type
 
-    
+
 class AttributeDefinition:
     """Construct representing an attribute of a `Table`"""
 
-    def __init__(self, name:str, type: attribute_type) -> None:
+    def __init__(self, name: str, type: attribute_type) -> None:
         """Create an attribute that will be part of a `Table`
-        
+
         Args:
             name (str): Name of the attribute
             type (attribute_type): Type of value the attribute stores
@@ -243,11 +258,9 @@ class AttributeDefinition:
         self.name = name
         self.type = type
 
-
     def render(self) -> attribute_definition_model:
         return attribute_definition_model(
-            attribute_name=self.name,
-            attribute_type=self.type
+            attribute_name=self.name, attribute_type=self.type
         )
 
 
@@ -258,48 +271,78 @@ class key_definition_model(ImmutableModel):
         attribute_name (str)
         key_type (`key_type`)
     """
-    attribute_name: str 
+
+    attribute_name: str
     key_type: key_type
 
+class secondary_key_model(ImmutableModel):
+    """Model representing a `Secondary Key`
+
+
+    Args:
+        index_name (str)
+        attribute_name (str))
+    """
+
+    index_name: str
+    attribute_name: str
 
 class KeyDefinition:
     """Construct representing a primary key of a `Table`"""
 
-    def __init__(self, name:str, type: key_type) -> None:
+    def __init__(self, name: str, type: key_type) -> None:
         """
         Args:
             name (str): Name of the attribute
             type (key_type): Type of value the attribute stores
 
         Note that the `name` property must match a `name` of a given `AttributeDefinition` on the created
-        table. 
+        table.
 
-        The keys on a table will define the optimal way of retrieving data from the `Table`. 
+        The keys on a table will define the optimal way of retrieving data from the `Table`.
 
 
         """
         self.name = name
         self.type = type
 
-
     def render(self) -> key_definition_model:
-        return key_definition_model(
-            attribute_name=self.name,
-            key_type=self.type
-        )
+        return key_definition_model(attribute_name=self.name, key_type=self.type)
+
+class SecondaryKeyDefinition:
+    """Construct representing a primary key of a `Table`"""
+
+    def __init__(self, index_name: str, attribute_name: str) -> None:
+        """
+        Args:
+            name (str): Name of the index
+            attribute_name (str): Name of the attribute that will be the secondary key
+
+        Note that the `name` property must match a `name` of a given `AttributeDefinition` on the created
+        table.
+
+        The keys on a table will define the optimal way of retrieving data from the `Table`.
 
 
-class simple_table_model(ResourceModel):
+        """
+        self.index_name = index_name
+        self.attribute_name = attribute_name
+
+    def render(self) -> secondary_key_model:
+        return secondary_key_model(index_name=self.index_name, attribute_name=self.attribute_name)
+
+class simple_table_model(TaggableResourceModel):
     """Model representing a `Table`
 
     Args:
         ResourceModel ([type]): [description]
     """
+
     attributes: FrozenSet[attribute_definition_model]
     keys: FrozenSet[key_definition_model]
+    secondary_key: FrozenSet[secondary_key_model]
 
-
-class Table(PermissionsAvailableMixin, Resource):
+class Table(PermissionsAvailableMixin, TaggableMixin, Resource):
     """Create a NoSql Table (DynamoDB)"""
 
     @update_hash
@@ -308,7 +351,9 @@ class Table(PermissionsAvailableMixin, Resource):
         cdev_name: str,
         attributes: List[AttributeDefinition],
         keys: List[KeyDefinition],
+        secondary_key: List[SecondaryKeyDefinition] =[],
         nonce: str = "",
+        tags: Dict[str, str] = None,
     ) -> None:
         """
         Args:
@@ -316,6 +361,7 @@ class Table(PermissionsAvailableMixin, Resource):
             attributes (List[AttributeDefinition]): List of Attributes on the Table
             keys (List[KeyDefinition]): List of Key Definitions that make up the primary keys
             nonce (str): Nonce to make the resource hash unique if there are conflicting resources with same configuration.
+            tags (Dict[str, str]): A set of tags to add to the resource
 
 
         <a href='https://code.tutsplus.com/tutorials/a-beginners-guide-to-http-and-rest--net-16340'>More information on how to use a DynamoDB Table</a>
@@ -326,10 +372,11 @@ class Table(PermissionsAvailableMixin, Resource):
 
         <a href="https://aws.amazon.com/dynamodb/pricing/"> Details on pricing</a>
         """
-        super().__init__(cdev_name, RUUID, nonce)
+        super().__init__(cdev_name, RUUID, nonce, tags=tags)
 
         self._attributes = attributes
         self._keys = keys
+        self._secondary_keys = secondary_key
         self._stream = None
 
         self._available_permissions: TablePermissions = TablePermissions(cdev_name)
@@ -362,9 +409,19 @@ class Table(PermissionsAvailableMixin, Resource):
 
     @keys.setter
     @update_hash
-    def keys(self, value: List[KeyDefinition]):
+    def keys(self, value: List[KeyDefinition]) -> None:
         self._keys = value
 
+    @property
+    def secondary_key(self) -> List[SecondaryKeyDefinition]:
+        """Current Secondary Keys"""
+        return self._secondary_keys
+
+    @secondary_key.setter
+    @update_hash
+    def secondary_key(self, value: List[SecondaryKeyDefinition]):
+        self._secondary_keys = value
+
     def create_stream(
         self, view_type: stream_type, batch_size: int = 100, batch_failure: bool = True
     ) -> StreamEvent:
@@ -382,17 +439,18 @@ class Table(PermissionsAvailableMixin, Resource):
             StreamEvent: The created `Stream_Event`
         """
         if self._stream:
-            raise Exception(f"Already created stream on this table. Use `get_stream()` to get the current stream.")
+            raise Exception(
+                f"Already created stream on this table. Use `get_stream()` to get the current stream."
+            )
 
-        event = StreamEvent(
+        self._stream = StreamEvent(
             table_name=self.name,
             view_type=view_type,
             batch_size=batch_size,
-            batch_failure=batch_failure
+            batch_failure=batch_failure,
         )
 
-        self._stream = event
-        return event
+        return self._stream
 
     def get_stream(self) -> StreamEvent:
         """Get the `StreamEvent` for this `Table`
@@ -404,18 +462,21 @@ class Table(PermissionsAvailableMixin, Resource):
             StreamEvent: The `StreamEvent` for this Table
         """
         if not self._stream:
-            raise Exception("Stream has not been created. Create a stream for this table using the `create_stream` function.")
+            raise Exception(
+                "Stream has not been created. Create a stream for this table using the `create_stream` function."
+            )
 
         return self._stream
 
-
-    def _check_attributes_and_keys(
-        self
-    ) -> None:
+    def _check_attributes_and_keys(self) -> None:
         """
         Check key constraints based on https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/dynamodb.html#DynamoDB.Client.create_table
         """
-        if len(self.keys) > 2:
+        key_count = len(self.keys)
+        if key_count == 0:
+            return
+
+        if key_count > 2:
             raise Exception("Only two primary keys can be used")
 
         primary_key = self.keys[0]
@@ -426,44 +487,44 @@ class Table(PermissionsAvailableMixin, Resource):
         if not primary_key.type == key_type.HASH:
             raise Exception("First key is not Hash key")
 
-        if not primary_key.name in set(
-            [x.name for x in self.attributes]
-        ):
+        attributes_names = set([x.name for x in self.attributes])
+        if primary_key.name not in attributes_names:
             raise Exception(
                 f"Hash key 'AttributeName' ({primary_key.name}) not defined in attributes",
             )
 
-        if len(self.keys) == 1:
-            return 
+        if key_count == 1:
+            return
 
         range_key = self.keys[1]
 
         if not range_key.type == key_type.RANGE:
             raise Exception("Second key is not a Range key")
 
-        if not range_key.name in set(
-            [x.name for x in self.attributes]
-        ):
+        if range_key.name not in attributes_names:
             raise Exception(
                 f"Range key 'AttributeName' ({range_key.name}) not defined in attributes",
             )
 
-    def compute_hash(self):
+    def compute_hash(self) -> None:
         self._hash = hasher.hash_list(
             [
-                [x.render() for x in self.attributes], 
-                [x.render() for x in self.keys], 
-                self.nonce
+                [x.render() for x in self.attributes],
+                [x.render() for x in self.keys],
+                self.nonce,
+                self._get_tags_hash(),
             ]
         )
 
     def render(self) -> simple_table_model:
         self._check_attributes_and_keys()
-        
+
         return simple_table_model(
             ruuid=RUUID,
             name=self.name,
             hash=self.hash,
             attributes=[x.render() for x in self.attributes],
             keys=[x.render() for x in self.keys],
+            secondary_key=[x.render() for x in self.secondary_key],
+            tags=frozendict(self.tags)
         )
